@@ -1,4 +1,4 @@
-/* $Id: cmd-break-pane.c 2553 2011-07-09 09:42:33Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -26,19 +26,19 @@
  * Break pane off into a window.
  */
 
-int	cmd_break_pane_exec(struct cmd *, struct cmd_ctx *);
+enum cmd_retval	 cmd_break_pane_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_break_pane_entry = {
 	"break-pane", "breakp",
-	"dt:", 0, 0,
-	"[-d] " CMD_TARGET_PANE_USAGE,
+	"dPF:t:", 0, 0,
+	"[-dP] [-F format] " CMD_TARGET_PANE_USAGE,
 	0,
 	NULL,
 	NULL,
 	cmd_break_pane_exec
 };
 
-int
+enum cmd_retval
 cmd_break_pane_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct args		*args = self->args;
@@ -46,15 +46,20 @@ cmd_break_pane_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct session		*s;
 	struct window_pane	*wp;
 	struct window		*w;
+	char			*name;
 	char			*cause;
 	int			 base_idx;
+	struct client		*c;
+	struct format_tree	*ft;
+	const char		*template;
+	char			*cp;
 
 	if ((wl = cmd_find_pane(ctx, args_get(args, 't'), &s, &wp)) == NULL)
-		return (-1);
+		return (CMD_RETURN_ERROR);
 
 	if (window_count_panes(wl->window) == 1) {
 		ctx->error(ctx, "can't break with only one pane");
-		return (-1);
+		return (CMD_RETURN_ERROR);
 	}
 
 	w = wl->window;
@@ -74,7 +79,9 @@ cmd_break_pane_exec(struct cmd *self, struct cmd_ctx *ctx)
 	w = wp->window = window_create1(s->sx, s->sy);
 	TAILQ_INSERT_HEAD(&w->panes, wp, entry);
 	w->active = wp;
-	w->name = default_window_name(w);
+	name = default_window_name(w);
+	window_set_name(w, name);
+	free(name);
 	layout_init(w);
 
 	base_idx = options_get_number(&s->options, "base-index");
@@ -85,5 +92,23 @@ cmd_break_pane_exec(struct cmd *self, struct cmd_ctx *ctx)
 	server_redraw_session(s);
 	server_status_session_group(s);
 
-	return (0);
+	if (args_has(args, 'P')) {
+
+		if ((template = args_get(args, 'F')) == NULL)
+			template = BREAK_PANE_TEMPLATE;
+
+		ft = format_create();
+		if ((c = cmd_find_client(ctx, NULL)) != NULL)
+			format_client(ft, c);
+		format_session(ft, s);
+		format_winlink(ft, s, wl);
+		format_window_pane(ft, wp);
+
+		cp = format_expand(ft, template);
+		ctx->print(ctx, "%s", cp);
+		free(cp);
+
+		format_free(ft);
+	}
+	return (CMD_RETURN_NORMAL);
 }
