@@ -1,4 +1,4 @@
-/* $Id: cmd-choose-client.c 2582 2011-08-21 12:38:55Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -19,6 +19,7 @@
 #include <sys/types.h>
 
 #include <ctype.h>
+#include <stdlib.h>
 
 #include "tmux.h"
 
@@ -26,15 +27,15 @@
  * Enter choice mode to choose a client.
  */
 
-int	cmd_choose_client_exec(struct cmd *, struct cmd_ctx *);
+enum cmd_retval	 cmd_choose_client_exec(struct cmd *, struct cmd_ctx *);
 
-void	cmd_choose_client_callback(void *, int);
-void	cmd_choose_client_free(void *);
+void	cmd_choose_client_callback(struct window_choose_data *);
+void	cmd_choose_client_free(struct window_choose_data *);
 
 const struct cmd_entry cmd_choose_client_entry = {
 	"choose-client", NULL,
-	"t:", 0, 1,
-	CMD_TARGET_WINDOW_USAGE " [template]",
+	"F:t:", 0, 1,
+	CMD_TARGET_WINDOW_USAGE " [-F format] [template]",
 	0,
 	NULL,
 	NULL,
@@ -46,25 +47,35 @@ struct cmd_choose_client_data {
 	char   		*template;
 };
 
-int
+enum cmd_retval
 cmd_choose_client_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct args			*args = self->args;
-	struct cmd_choose_client_data	*cdata;
+	struct window_choose_data	*cdata;
 	struct winlink			*wl;
 	struct client			*c;
+	const char			*template;
+	char				*action;
 	u_int			 	 i, idx, cur;
 
 	if (ctx->curclient == NULL) {
 		ctx->error(ctx, "must be run interactively");
-		return (-1);
+		return (CMD_RETURN_ERROR);
 	}
 
 	if ((wl = cmd_find_window(ctx, args_get(args, 't'), NULL)) == NULL)
-		return (-1);
+		return (CMD_RETURN_ERROR);
 
 	if (window_pane_set_mode(wl->window->active, &window_choose_mode) != 0)
-		return (0);
+		return (CMD_RETURN_NORMAL);
+
+	if ((template = args_get(args, 'F')) == NULL)
+		template = CHOOSE_CLIENT_TEMPLATE;
+
+	if (args->argc != 0)
+		action = xstrdup(args->argv[0]);
+	else
+		action = xstrdup("detach-client -t '%%'");
 
 	cur = idx = 0;
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
@@ -75,79 +86,56 @@ cmd_choose_client_exec(struct cmd *self, struct cmd_ctx *ctx)
 			cur = idx;
 		idx++;
 
-		window_choose_add(wl->window->active, i,
-		    "%s: %s [%ux%u %s]%s%s", c->tty.path,
-		    c->session->name, c->tty.sx, c->tty.sy,
-		    c->tty.termname,
-		    c->tty.flags & TTY_UTF8 ? " (utf8)" : "",
-		    c->flags & CLIENT_READONLY ? " (ro)" : "");
-	}
+		cdata = window_choose_data_create(ctx);
+		cdata->idx = i;
+		cdata->client->references++;
 
-	cdata = xmalloc(sizeof *cdata);
-	if (args->argc != 0)
-		cdata->template = xstrdup(args->argv[0]);
-	else
-		cdata->template = xstrdup("detach-client -t '%%'");
-	cdata->client = ctx->curclient;
-	cdata->client->references++;
+		cdata->ft_template = xstrdup(template);
+		format_add(cdata->ft, "line", "%u", i);
+		format_session(cdata->ft, c->session);
+		format_client(cdata->ft, c);
+
+		cdata->command = cmd_template_replace(action, c->tty.path, 1);
+
+		window_choose_add(wl->window->active, cdata);
+	}
+	free(action);
 
 	window_choose_ready(wl->window->active,
-	    cur, cmd_choose_client_callback, cmd_choose_client_free, cdata);
+	    cur, cmd_choose_client_callback, cmd_choose_client_free);
 
-	return (0);
+	return (CMD_RETURN_NORMAL);
 }
 
 void
-cmd_choose_client_callback(void *data, int idx)
+cmd_choose_client_callback(struct window_choose_data *cdata)
 {
-	struct cmd_choose_client_data	*cdata = data;
-	struct client  			*c;
-	struct cmd_list			*cmdlist;
-	struct cmd_ctx			 ctx;
-	char				*template, *cause;
+	struct client  	*c;
 
-	if (idx == -1)
+	if (cdata == NULL)
 		return;
 	if (cdata->client->flags & CLIENT_DEAD)
 		return;
 
-	if ((u_int) idx > ARRAY_LENGTH(&clients) - 1)
+	if (cdata->idx > ARRAY_LENGTH(&clients) - 1)
 		return;
-	c = ARRAY_ITEM(&clients, idx);
+	c = ARRAY_ITEM(&clients, cdata->idx);
 	if (c == NULL || c->session == NULL)
 		return;
-	template = cmd_template_replace(cdata->template, c->tty.path, 1);
 
-	if (cmd_string_parse(template, &cmdlist, &cause) != 0) {
-		if (cause != NULL) {
-			*cause = toupper((u_char) *cause);
-			status_message_set(c, "%s", cause);
-			xfree(cause);
-		}
-		xfree(template);
-		return;
-	}
-	xfree(template);
-
-	ctx.msgdata = NULL;
-	ctx.curclient = cdata->client;
-
-	ctx.error = key_bindings_error;
-	ctx.print = key_bindings_print;
-	ctx.info = key_bindings_info;
-
-	ctx.cmdclient = NULL;
-
-	cmd_list_exec(cmdlist, &ctx);
-	cmd_list_free(cmdlist);
+	window_choose_ctx(cdata);
 }
 
 void
-cmd_choose_client_free(void *data)
+cmd_choose_client_free(struct window_choose_data *cdata)
 {
-	struct cmd_choose_client_data	*cdata = data;
+	if (cdata == NULL)
+		return;
 
 	cdata->client->references--;
-	xfree(cdata->template);
-	xfree(cdata);
+
+	free(cdata->ft_template);
+	free(cdata->command);
+	format_free(cdata->ft);
+	free(cdata);
 }
