@@ -1,4 +1,4 @@
-/* $Id: cmd-new-window.c 2664 2012-01-20 21:21:32Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -30,28 +30,33 @@ int	cmd_new_window_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_new_window_entry = {
 	"new-window", "neww",
-	"adkn:Pt:", 0, 1,
-	"[-adk] [-n window-name] [-t target-window] [command]",
+	"ac:dF:kn:Pt:", 0, 1,
+	"[-adkP] [-c start-directory] [-F format] [-n window-name] "
+	"[-t target-window] [command]",
 	0,
 	NULL,
 	NULL,
 	cmd_new_window_exec
 };
 
-int
+enum cmd_retval
 cmd_new_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	struct args	*args = self->args;
-	struct session	*s;
-	struct winlink	*wl;
-	const char     	*cmd, *cwd;
-	char		*cause;
-	int		 idx, last, detached;
+	struct args		*args = self->args;
+	struct session		*s;
+	struct winlink		*wl;
+	struct client		*c;
+	const char		*cmd, *cwd;
+	const char		*template;
+	char			*cause;
+	int			 idx, last, detached;
+	struct format_tree	*ft;
+	char			*cp;
 
 	if (args_has(args, 'a')) {
 		wl = cmd_find_window(ctx, args_get(args, 't'), &s);
 		if (wl == NULL)
-			return (-1);
+			return (CMD_RETURN_ERROR);
 		idx = wl->idx + 1;
 
 		/* Find the next free index. */
@@ -61,7 +66,7 @@ cmd_new_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		}
 		if (last == INT_MAX) {
 			ctx->error(ctx, "no free window indexes");
-			return (-1);
+			return (CMD_RETURN_ERROR);
 		}
 
 		/* Move everything from last - 1 to idx up a bit. */
@@ -72,7 +77,7 @@ cmd_new_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		}
 	} else {
 		if ((idx = cmd_find_index(ctx, args_get(args, 't'), &s)) == -2)
-			return (-1);
+			return (CMD_RETURN_ERROR);
 	}
 	detached = args_has(args, 'd');
 
@@ -84,6 +89,7 @@ cmd_new_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		 * Can't use session_detach as it will destroy session if this
 		 * makes it empty.
 		 */
+		notify_window_unlinked(s, wl->window);
 		wl->flags &= ~WINLINK_ALERTFLAGS;
 		winlink_stack_remove(&s->lastw, wl);
 		winlink_remove(&s->windows, wl);
@@ -99,15 +105,15 @@ cmd_new_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		cmd = options_get_string(&s->options, "default-command");
 	else
 		cmd = args->argv[0];
-	cwd = cmd_get_default_path(ctx);
+	cwd = cmd_get_default_path(ctx, args_get(args, 'c'));
 
 	if (idx == -1)
 		idx = -1 - options_get_number(&s->options, "base-index");
 	wl = session_new(s, args_get(args, 'n'), cmd, cwd, idx, &cause);
 	if (wl == NULL) {
 		ctx->error(ctx, "create window failed: %s", cause);
-		xfree(cause);
-		return (-1);
+		free(cause);
+		return (CMD_RETURN_ERROR);
 	}
 	if (!detached) {
 		session_select(s, wl->idx);
@@ -115,7 +121,23 @@ cmd_new_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	} else
 		server_status_session_group(s);
 
-	if (args_has(args, 'P'))
-		ctx->print(ctx, "%s:%u", s->name, wl->idx);
-	return (0);
+	if (args_has(args, 'P')) {
+		if ((template = args_get(args, 'F')) == NULL)
+			template = NEW_WINDOW_TEMPLATE;
+
+		ft = format_create();
+		if ((c = cmd_find_client(ctx, NULL)) != NULL)
+		    format_client(ft, c);
+		format_session(ft, s);
+		format_winlink(ft, s, wl);
+		format_window_pane(ft, wl->window->active);
+
+		cp = format_expand(ft, template);
+		ctx->print(ctx, "%s", cp);
+		free(cp);
+
+		format_free(ft);
+	}
+
+	return (CMD_RETURN_NORMAL);
 }
