@@ -1,4 +1,4 @@
-/* $Id: cmd-paste-buffer.c 2553 2011-07-09 09:42:33Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -27,22 +27,22 @@
  * Paste paste buffer if present.
  */
 
-int	cmd_paste_buffer_exec(struct cmd *, struct cmd_ctx *);
+enum cmd_retval	 cmd_paste_buffer_exec(struct cmd *, struct cmd_ctx *);
 
-void	cmd_paste_buffer_filter(
-	    struct window_pane *, const char *, size_t, const char *);
+void	cmd_paste_buffer_filter(struct window_pane *,
+	    const char *, size_t, const char *, int bracket);
 
 const struct cmd_entry cmd_paste_buffer_entry = {
 	"paste-buffer", "pasteb",
-	"db:rs:t:", 0, 0,
-	"[-dr] [-s separator] [-b buffer-index] [-t target-pane]",
+	"db:prs:t:", 0, 0,
+	"[-dpr] [-s separator] [-b buffer-index] [-t target-pane]",
 	0,
 	NULL,
 	NULL,
 	cmd_paste_buffer_exec
 };
 
-int
+enum cmd_retval
 cmd_paste_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct args		*args = self->args;
@@ -52,9 +52,10 @@ cmd_paste_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 	const char		*sepstr;
 	char			*cause;
 	int			 buffer;
+	int			 pflag;
 
 	if (cmd_find_pane(ctx, args_get(args, 't'), &s, &wp) == NULL)
-		return (-1);
+		return (CMD_RETURN_ERROR);
 
 	if (!args_has(args, 'b'))
 		buffer = -1;
@@ -62,8 +63,8 @@ cmd_paste_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 		buffer = args_strtonum(args, 'b', 0, INT_MAX, &cause);
 		if (cause != NULL) {
 			ctx->error(ctx, "buffer %s", cause);
-			xfree(cause);
-			return (-1);
+			free(cause);
+			return (CMD_RETURN_ERROR);
 		}
 	}
 
@@ -73,7 +74,7 @@ cmd_paste_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 		pb = paste_get_index(&global_buffers, buffer);
 		if (pb == NULL) {
 			ctx->error(ctx, "no buffer %d", buffer);
-			return (-1);
+			return (CMD_RETURN_ERROR);
 		}
 	}
 
@@ -85,7 +86,9 @@ cmd_paste_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 			else
 				sepstr = "\r";
 		}
-		cmd_paste_buffer_filter(wp, pb->data, pb->size, sepstr);
+		pflag = args_has(args, 'p') &&
+		    (wp->screen->mode & MODE_BRACKETPASTE);
+		cmd_paste_buffer_filter(wp, pb->data, pb->size, sepstr, pflag);
 	}
 
 	/* Delete the buffer if -d. */
@@ -96,18 +99,21 @@ cmd_paste_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 			paste_free_index(&global_buffers, buffer);
 	}
 
-	return (0);
+	return (CMD_RETURN_NORMAL);
 }
 
 /* Add bytes to a buffer and filter '\n' according to separator. */
 void
-cmd_paste_buffer_filter(
-    struct window_pane *wp, const char *data, size_t size, const char *sep)
+cmd_paste_buffer_filter(struct window_pane *wp,
+    const char *data, size_t size, const char *sep, int bracket)
 {
 	const char	*end = data + size;
 	const char	*lf;
 	size_t		 seplen;
 
+	if (bracket)
+		bufferevent_write(wp->event, "\033[200~", 6);
+
 	seplen = strlen(sep);
 	while ((lf = memchr(data, '\n', end - data)) != NULL) {
 		if (lf != data)
@@ -118,4 +124,7 @@ cmd_paste_buffer_filter(
 
 	if (end != data)
 		bufferevent_write(wp->event, data, end - data);
+
+	if (bracket)
+		bufferevent_write(wp->event, "\033[201~", 6);
 }
