@@ -1,4 +1,4 @@
-/* $Id: cmd-server-info.c 2553 2011-07-09 09:42:33Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -30,7 +30,7 @@
  * Show various information about server.
  */
 
-int	cmd_server_info_exec(struct cmd *, struct cmd_ctx *);
+enum cmd_retval	 cmd_server_info_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_server_info_entry = {
 	"server-info", "info",
@@ -43,7 +43,7 @@ const struct cmd_entry cmd_server_info_entry = {
 };
 
 /* ARGSUSED */
-int
+enum cmd_retval
 cmd_server_info_exec(unused struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct tty_term				*term;
@@ -88,12 +88,12 @@ cmd_server_info_exec(unused struct cmd *self, struct cmd_ctx *ctx)
 		if (c == NULL || c->session == NULL)
 			continue;
 
-		ctx->print(ctx,"%2d: %s (%d, %d): %s [%ux%u %s bs=%hho] "
-		    "[flags=0x%x/0x%x, references=%u]", i, c->tty.path,
-		    c->ibuf.fd, c->tty.fd, c->session->name,
+		ctx->print(ctx,"%2d: %s (%d, %d): %s [%ux%u %s bs=%hho "
+		    "xterm=%u] [flags=0x%x/0x%x, references=%u]", i,
+		    c->tty.path, c->ibuf.fd, c->tty.fd, c->session->name,
 		    c->tty.sx, c->tty.sy, c->tty.termname,
-		    c->tty.tio.c_cc[VERASE], c->flags,
-		    c->tty.flags, c->references);
+		    c->tty.tio.c_cc[VERASE], c->tty.xterm_version,
+		    c->flags, c->tty.flags, c->references);
 	}
 	ctx->print(ctx, "%s", "");
 
@@ -179,5 +179,5 @@ cmd_server_info_exec(unused struct cmd *self, struct cmd_ctx *ctx)
 		    job->cmd, job->fd, job->pid, job->status);
 	}
 
-	return (0);
+	return (CMD_RETURN_NORMAL);
 }
