@@ -1,4 +1,4 @@
-/* $Id: cmd-show-buffer.c 2616 2011-10-23 15:02:20Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,13 +18,15 @@
 
 #include <sys/types.h>
 
+#include <stdlib.h>
+
 #include "tmux.h"
 
 /*
  * Show a paste buffer.
  */
 
-int	cmd_show_buffer_exec(struct cmd *, struct cmd_ctx *);
+enum cmd_retval	 cmd_show_buffer_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_show_buffer_entry = {
 	"show-buffer", "showb",
@@ -36,7 +38,7 @@ const struct cmd_entry cmd_show_buffer_entry = {
 	cmd_show_buffer_exec
 };
 
-int
+enum cmd_retval
 cmd_show_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct args		*args = self->args;
@@ -48,25 +50,25 @@ cmd_show_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 	u_int			 width;
 
 	if ((s = cmd_find_session(ctx, NULL, 0)) == NULL)
-		return (-1);
+		return (CMD_RETURN_ERROR);
 
 	if (!args_has(args, 'b')) {
 		if ((pb = paste_get_top(&global_buffers)) == NULL) {
 			ctx->error(ctx, "no buffers");
-			return (-1);
+			return (CMD_RETURN_ERROR);
 		}
 	} else {
 		buffer = args_strtonum(args, 'b', 0, INT_MAX, &cause);
 		if (cause != NULL) {
 			ctx->error(ctx, "buffer %s", cause);
-			xfree(cause);
-			return (-1);
+			free(cause);
+			return (CMD_RETURN_ERROR);
 		}
 
 		pb = paste_get_index(&global_buffers, buffer);
 		if (pb == NULL) {
 			ctx->error(ctx, "no buffer %d", buffer);
-			return (-1);
+			return (CMD_RETURN_ERROR);
 		}
 	}
 
@@ -101,9 +103,9 @@ cmd_show_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 		buf[len] = '\0';
 		ctx->print(ctx, "%s", buf);
 	}
-	xfree(buf);
+	free(buf);
 
-	xfree(in);
+	free(in);
 
-	return (0);
+	return (CMD_RETURN_NORMAL);
 }
