@@ -1,4 +1,4 @@
-/* $Id: cmd-split-window.c 2664 2012-01-20 21:21:32Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -27,13 +27,14 @@
  * Split a window (add a new pane).
  */
 
-void	cmd_split_window_key_binding(struct cmd *, int);
-int	cmd_split_window_exec(struct cmd *, struct cmd_ctx *);
+void		 cmd_split_window_key_binding(struct cmd *, int);
+enum cmd_retval	 cmd_split_window_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_split_window_entry = {
 	"split-window", "splitw",
-	"dl:hp:Pt:v", 0, 1,
-	"[-dhvP] [-p percentage|-l size] [-t target-pane] [command]",
+	"c:dF:l:hp:Pt:v", 0, 1,
+	"[-dhvP] [-c start-directory] [-F format] [-p percentage|-l size] "
+	"[-t target-pane] [command]",
 	0,
 	cmd_split_window_key_binding,
 	NULL,
@@ -48,7 +49,7 @@ cmd_split_window_key_binding(struct cmd *self, int key)
 		args_set(self->args, 'h', NULL);
 }
 
-int
+enum cmd_retval
 cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct args		*args = self->args;
@@ -57,15 +58,19 @@ cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct window		*w;
 	struct window_pane	*wp, *new_wp = NULL;
 	struct environ		 env;
-	const char	       	*cmd, *cwd, *shell;
+	const char		*cmd, *cwd, *shell;
 	char			*cause, *new_cause;
-	u_int			 hlimit, paneidx;
+	u_int			 hlimit;
 	int			 size, percentage;
 	enum layout_type	 type;
 	struct layout_cell	*lc;
+	const char		*template;
+	struct client		*c;
+	struct format_tree	*ft;
+	char			*cp;
 
 	if ((wl = cmd_find_pane(ctx, args_get(args, 't'), &s, &wp)) == NULL)
-		return (-1);
+		return (CMD_RETURN_ERROR);
 	w = wl->window;
 
 	environ_init(&env);
@@ -77,7 +82,7 @@ cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		cmd = options_get_string(&s->options, "default-command");
 	else
 		cmd = args->argv[0];
-	cwd = cmd_get_default_path(ctx);
+	cwd = cmd_get_default_path(ctx, args_get(args, 'c'));
 
 	type = LAYOUT_TOPBOTTOM;
 	if (args_has(args, 'h'))
@@ -88,7 +93,7 @@ cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		size = args_strtonum(args, 'l', 0, INT_MAX, &cause);
 		if (cause != NULL) {
 			xasprintf(&new_cause, "size %s", cause);
-			xfree(cause);
+			free(cause);
 			cause = new_cause;
 			goto error;
 		}
@@ -96,7 +101,7 @@ cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		percentage = args_strtonum(args, 'p', 0, INT_MAX, &cause);
 		if (cause != NULL) {
 			xasprintf(&new_cause, "percentage %s", cause);
-			xfree(cause);
+			free(cause);
 			cause = new_cause;
 			goto error;
 		}
@@ -111,7 +116,7 @@ cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	if (*shell == '\0' || areshell(shell))
 		shell = _PATH_BSHELL;
 
-	if ((lc = layout_split_pane(wp, type, size)) == NULL) {
+	if ((lc = layout_split_pane(wp, type, size, 0)) == NULL) {
 		cause = xstrdup("pane too small");
 		goto error;
 	}
@@ -133,17 +138,30 @@ cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	environ_free(&env);
 
 	if (args_has(args, 'P')) {
-		if (window_pane_index(new_wp, &paneidx) != 0)
-			fatalx("index not found");
-		ctx->print(ctx, "%s:%u.%u", s->name, wl->idx, paneidx);
+		if ((template = args_get(args, 'F')) == NULL)
+			template = SPLIT_WINDOW_TEMPLATE;
+
+		ft = format_create();
+		if ((c = cmd_find_client(ctx, NULL)) != NULL)
+		    format_client(ft, c);
+		format_session(ft, s);
+		format_winlink(ft, s, wl);
+		format_window_pane(ft, new_wp);
+
+		cp = format_expand(ft, template);
+		ctx->print(ctx, "%s", cp);
+		free(cp);
+
+		format_free(ft);
 	}
-	return (0);
+	notify_window_layout_changed(w);
+	return (CMD_RETURN_NORMAL);
 
 error:
 	environ_free(&env);
 	if (new_wp != NULL)
 		window_remove_pane(w, new_wp);
 	ctx->error(ctx, "create pane failed: %s", cause);
-	xfree(cause);
-	return (-1);
+	free(cause);
+	return (CMD_RETURN_ERROR);
 }
