@@ -1,4 +1,4 @@
-/* $Id: cmd-unbind-key.c 2670 2012-01-21 19:38:26Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,16 +18,17 @@
 
 #include <sys/types.h>
 
+#include <stdlib.h>
+
 #include "tmux.h"
 
 /*
  * Unbind key from command.
  */
 
-int	cmd_unbind_key_check(struct args *);
-int	cmd_unbind_key_exec(struct cmd *, struct cmd_ctx *);
-
-int	cmd_unbind_key_table(struct cmd *, struct cmd_ctx *, int);
+enum cmd_retval	 cmd_unbind_key_check(struct args *);
+enum cmd_retval	 cmd_unbind_key_exec(struct cmd *, struct cmd_ctx *);
+enum cmd_retval	 cmd_unbind_key_table(struct cmd *, struct cmd_ctx *, int);
 
 const struct cmd_entry cmd_unbind_key_entry = {
 	"unbind-key", "unbind",
@@ -39,47 +40,50 @@ const struct cmd_entry cmd_unbind_key_entry = {
 	cmd_unbind_key_exec
 };
 
-int
+enum cmd_retval
 cmd_unbind_key_check(struct args *args)
 {
-	if (args_has(args, 'a') && (args->argc != 0 || args_has(args, 't')))
-	    return (-1);
+	if (args_has(args, 'a') && args->argc != 0)
+		return (CMD_RETURN_ERROR);
 	if (!args_has(args, 'a') && args->argc != 1)
-		return (-1);
-	return (0);
+		return (CMD_RETURN_ERROR);
+	return (CMD_RETURN_NORMAL);
 }
 
-int
+enum cmd_retval
 cmd_unbind_key_exec(struct cmd *self, unused struct cmd_ctx *ctx)
 {
 	struct args		*args = self->args;
 	struct key_binding	*bd;
 	int			 key;
 
-	if (args_has(args, 'a')) {
-		while (!RB_EMPTY(&key_bindings)) {
-			bd = RB_ROOT(&key_bindings);
-			key_bindings_remove(bd->key);
+	if (!args_has(args, 'a')) {
+		key = key_string_lookup_string(args->argv[0]);
+		if (key == KEYC_NONE) {
+			ctx->error(ctx, "unknown key: %s", args->argv[0]);
+			return (CMD_RETURN_ERROR);
 		}
-		return (0);
-	}
-
-	key = key_string_lookup_string(args->argv[0]);
-	if (key == KEYC_NONE) {
-		ctx->error(ctx, "unknown key: %s", args->argv[0]);
-		return (-1);
-	}
+	} else
+		key = KEYC_NONE;
 
 	if (args_has(args, 't'))
 		return (cmd_unbind_key_table(self, ctx, key));
 
+	if (key == KEYC_NONE) {
+		while (!RB_EMPTY(&key_bindings)) {
+			bd = RB_ROOT(&key_bindings);
+			key_bindings_remove(bd->key);
+		}
+		return (CMD_RETURN_NORMAL);
+	}
+
 	if (!args_has(args, 'n'))
 		key |= KEYC_PREFIX;
 	key_bindings_remove(key);
-	return (0);
+	return (CMD_RETURN_NORMAL);
 }
 
-int
+enum cmd_retval
 cmd_unbind_key_table(struct cmd *self, struct cmd_ctx *ctx, int key)
 {
 	struct args			*args = self->args;
@@ -90,14 +94,23 @@ cmd_unbind_key_table(struct cmd *self, struct cmd_ctx *ctx, int key)
 	tablename = args_get(args, 't');
 	if ((mtab = mode_key_findtable(tablename)) == NULL) {
 		ctx->error(ctx, "unknown key table: %s", tablename);
-		return (-1);
+		return (CMD_RETURN_ERROR);
+	}
+
+	if (key == KEYC_NONE) {
+		while (!RB_EMPTY(mtab->tree)) {
+			mbind = RB_ROOT(mtab->tree);
+			RB_REMOVE(mode_key_tree, mtab->tree, mbind);
+			free(mbind);
+		}
+		return (CMD_RETURN_NORMAL);
 	}
 
 	mtmp.key = key;
 	mtmp.mode = !!args_has(args, 'c');
 	if ((mbind = RB_FIND(mode_key_tree, mtab->tree, &mtmp)) != NULL) {
 		RB_REMOVE(mode_key_tree, mtab->tree, mbind);
-		xfree(mbind);
+		free(mbind);
 	}
-	return (0);
+	return (CMD_RETURN_NORMAL);
 }
