@@ -1,4 +1,4 @@
-/* $Id: cmd.c 2664 2012-01-20 21:21:32Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -20,6 +20,7 @@
 #include <sys/time.h>
 
 #include <fnmatch.h>
+#include <pwd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -33,7 +34,9 @@ const struct cmd_entry *cmd_table[] = {
 	&cmd_capture_pane_entry,
 	&cmd_choose_buffer_entry,
 	&cmd_choose_client_entry,
+	&cmd_choose_list_entry,
 	&cmd_choose_session_entry,
+	&cmd_choose_tree_entry,
 	&cmd_choose_window_entry,
 	&cmd_clear_history_entry,
 	&cmd_clock_mode_entry,
@@ -66,6 +69,7 @@ const struct cmd_entry *cmd_table[] = {
 	&cmd_lock_client_entry,
 	&cmd_lock_server_entry,
 	&cmd_lock_session_entry,
+	&cmd_move_pane_entry,
 	&cmd_move_window_entry,
 	&cmd_new_session_entry,
 	&cmd_new_window_entry,
@@ -119,8 +123,10 @@ struct session	*cmd_lookup_session(const char *, int *);
 struct winlink	*cmd_lookup_window(struct session *, const char *, int *);
 int		 cmd_lookup_index(struct session *, const char *, int *);
 struct window_pane *cmd_lookup_paneid(const char *);
-struct session	*cmd_pane_session(struct cmd_ctx *,
-		    struct window_pane *, struct winlink **);
+struct winlink	*cmd_lookup_winlink_windowid(struct session *, const char *);
+struct window	*cmd_lookup_windowid(const char *);
+struct session	*cmd_window_session(struct cmd_ctx *,
+		    struct window *, struct winlink **);
 struct winlink	*cmd_find_window_offset(const char *, struct session *, int *);
 int		 cmd_find_index_offset(const char *, struct session *, int *);
 struct window_pane *cmd_find_pane_offset(const char *, struct winlink *);
@@ -192,11 +198,9 @@ cmd_free_argv(int argc, char **argv)
 
 	if (argc == 0)
 		return;
-	for (i = 0; i < argc; i++) {
-		if (argv[i] != NULL)
-			xfree(argv[i]);
-	}
-	xfree(argv);
+	for (i = 0; i < argc; i++)
+		free(argv[i]);
+	free(argv);
 }
 
 struct cmd *
@@ -276,7 +280,7 @@ usage:
 	return (NULL);
 }
 
-int
+enum cmd_retval
 cmd_exec(struct cmd *cmd, struct cmd_ctx *ctx)
 {
 	return (cmd->entry->exec(cmd, ctx));
@@ -285,9 +289,8 @@ cmd_exec(struct cmd *cmd, struct cmd_ctx *ctx)
 void
 cmd_free(struct cmd *cmd)
 {
-	if (cmd->args != NULL)
-		args_free(cmd->args);
-	xfree(cmd);
+	args_free(cmd->args);
+	free(cmd);
 }
 
 size_t
@@ -299,11 +302,10 @@ cmd_print(struct cmd *cmd, char *buf, size_t len)
 	if (off < len) {
 		used = args_print(cmd->args, buf + off, len - off);
 		if (used == 0)
-			buf[off - 1] = '\0';
-		else {
+			off--;
+		else
 			off += used;
-			buf[off] = '\0';
-		}
+		buf[off] = '\0';
 	}
 	return (off);
 }
@@ -502,7 +504,7 @@ cmd_find_client(struct cmd_ctx *ctx, const char *arg)
 	if (c == NULL)
 		ctx->error(ctx, "client not found: %s", tmparg);
 
-	xfree(tmparg);
+	free(tmparg);
 	return (c);
 }
 
@@ -586,6 +588,10 @@ cmd_lookup_window(struct session *s, const char *name, int *ambiguous)
 
 	*ambiguous = 0;
 
+	/* Try as a window id. */
+	if ((wl = cmd_lookup_winlink_windowid(s, name)) != NULL)
+	    return (wl);
+
 	/* First see if this is a valid window index in this session. */
 	idx = strtonum(name, 0, INT_MAX, &errstr);
 	if (errstr == NULL) {
@@ -648,10 +654,7 @@ cmd_lookup_index(struct session *s, const char *name, int *ambiguous)
 	return (-1);
 }
 
-/*
- * Lookup pane id. An initial % means a pane id. sp must already point to the
- * current session.
- */
+/* Lookup pane id. An initial % means a pane id. */
 struct window_pane *
 cmd_lookup_paneid(const char *arg)
 {
@@ -667,19 +670,50 @@ cmd_lookup_paneid(const char *arg)
 	return (window_pane_find_by_id(paneid));
 }
 
-/* Find session and winlink for pane. */
+/* Lookup window id in a session. An initial @ means a window id. */
+struct winlink *
+cmd_lookup_winlink_windowid(struct session *s, const char *arg)
+{
+	const char	*errstr;
+	u_int		 windowid;
+
+	if (*arg != '@')
+		return (NULL);
+
+	windowid = strtonum(arg + 1, 0, UINT_MAX, &errstr);
+	if (errstr != NULL)
+		return (NULL);
+	return (winlink_find_by_window_id(&s->windows, windowid));
+}
+
+/* Lookup window id. An initial @ means a window id. */
+struct window *
+cmd_lookup_windowid(const char *arg)
+{
+	const char	*errstr;
+	u_int		 windowid;
+
+	if (*arg != '@')
+		return (NULL);
+
+	windowid = strtonum(arg + 1, 0, UINT_MAX, &errstr);
+	if (errstr != NULL)
+		return (NULL);
+	return (window_find_by_id(windowid));
+}
+
+/* Find session and winlink for window. */
 struct session *
-cmd_pane_session(struct cmd_ctx *ctx, struct window_pane *wp,
-    struct winlink **wlp)
+cmd_window_session(struct cmd_ctx *ctx, struct window *w, struct winlink **wlp)
 {
 	struct session		*s;
 	struct sessionslist	 ss;
 	struct winlink		*wl;
 
-	/* If this pane is in the current session, return that winlink. */
+	/* If this window is in the current session, return that winlink. */
 	s = cmd_current_session(ctx, 0);
 	if (s != NULL) {
-		wl = winlink_find_by_window(&s->windows, wp->window);
+		wl = winlink_find_by_window(&s->windows, w);
 		if (wl != NULL) {
 			if (wlp != NULL)
 				*wlp = wl;
@@ -687,16 +721,16 @@ cmd_pane_session(struct cmd_ctx *ctx, struct window_pane *wp,
 		}
 	}
 
-	/* Otherwise choose from all sessions with this pane. */
+	/* Otherwise choose from all sessions with this window. */
 	ARRAY_INIT(&ss);
 	RB_FOREACH(s, sessions, &sessions) {
-		if (winlink_find_by_window(&s->windows, wp->window) != NULL)
+		if (winlink_find_by_window(&s->windows, w) != NULL)
 			ARRAY_ADD(&ss, s);
 	}
 	s = cmd_choose_session_list(&ss);
 	ARRAY_FREE(&ss);
 	if (wlp != NULL)
-		*wlp = winlink_find_by_window(&s->windows, wp->window);
+		*wlp = winlink_find_by_window(&s->windows, w);
 	return (s);
 }
 
@@ -706,6 +740,7 @@ cmd_find_session(struct cmd_ctx *ctx, const char *arg, int prefer_unattached)
 {
 	struct session		*s;
 	struct window_pane	*wp;
+	struct window		*w;
 	struct client		*c;
 	char			*tmparg;
 	size_t			 arglen;
@@ -715,9 +750,11 @@ cmd_find_session(struct cmd_ctx *ctx, const char *arg, int prefer_unattached)
 	if (arg == NULL)
 		return (cmd_current_session(ctx, prefer_unattached));
 
-	/* Lookup as pane id. */
+	/* Lookup as pane id or window id. */
 	if ((wp = cmd_lookup_paneid(arg)) != NULL)
-		return (cmd_pane_session(ctx, wp, NULL));
+		return (cmd_window_session(ctx, wp->window, NULL));
+	if ((w = cmd_lookup_windowid(arg)) != NULL)
+		return (cmd_window_session(ctx, w, NULL));
 
 	/* Trim a single trailing colon if any. */
 	tmparg = xstrdup(arg);
@@ -727,7 +764,7 @@ cmd_find_session(struct cmd_ctx *ctx, const char *arg, int prefer_unattached)
 
 	/* An empty session name is the current session. */
 	if (*tmparg == '\0') {
-		xfree(tmparg);
+		free(tmparg);
 		return (cmd_current_session(ctx, prefer_unattached));
 	}
 
@@ -746,7 +783,7 @@ cmd_find_session(struct cmd_ctx *ctx, const char *arg, int prefer_unattached)
 			ctx->error(ctx, "session not found: %s", tmparg);
 	}
 
-	xfree(tmparg);
+	free(tmparg);
 	return (s);
 }
 
@@ -779,7 +816,7 @@ cmd_find_window(struct cmd_ctx *ctx, const char *arg, struct session **sp)
 
 	/* Lookup as pane id. */
 	if ((wp = cmd_lookup_paneid(arg)) != NULL) {
-		s = cmd_pane_session(ctx, wp, &wl);
+		s = cmd_window_session(ctx, wp->window, &wl);
 		if (sp != NULL)
 			*sp = s;
 		return (wl);
@@ -821,7 +858,7 @@ cmd_find_window(struct cmd_ctx *ctx, const char *arg, struct session **sp)
 		goto not_found;
 
 	if (sessptr != NULL)
-		xfree(sessptr);
+		free(sessptr);
 	return (wl);
 
 no_colon:
@@ -859,8 +896,7 @@ no_session:
 		ctx->error(ctx, "multiple sessions: %s", arg);
 	else
 		ctx->error(ctx, "session not found: %s", arg);
-	if (sessptr != NULL)
-		xfree(sessptr);
+	free(sessptr);
 	return (NULL);
 
 not_found:
@@ -868,8 +904,7 @@ not_found:
 		ctx->error(ctx, "multiple windows: %s", arg);
 	else
 		ctx->error(ctx, "window not found: %s", arg);
-	if (sessptr != NULL)
-		xfree(sessptr);
+	free(sessptr);
 	return (NULL);
 }
 
@@ -960,8 +995,7 @@ cmd_find_index(struct cmd_ctx *ctx, const char *arg, struct session **sp)
 	} else if ((idx = cmd_lookup_index(s, winptr, &ambiguous)) == -1)
 		goto invalid_index;
 
-	if (sessptr != NULL)
-		xfree(sessptr);
+	free(sessptr);
 	return (idx);
 
 no_colon:
@@ -1000,8 +1034,7 @@ no_session:
 		ctx->error(ctx, "multiple sessions: %s", arg);
 	else
 		ctx->error(ctx, "session not found: %s", arg);
-	if (sessptr != NULL)
-		xfree(sessptr);
+	free(sessptr);
 	return (-2);
 
 invalid_index:
@@ -1009,8 +1042,7 @@ invalid_index:
 		goto not_found;
 	ctx->error(ctx, "invalid index: %s", arg);
 
-	if (sessptr != NULL)
-		xfree(sessptr);
+	free(sessptr);
 	return (-2);
 
 not_found:
@@ -1018,8 +1050,7 @@ not_found:
 		ctx->error(ctx, "multiple windows: %s", arg);
 	else
 		ctx->error(ctx, "window not found: %s", arg);
-	if (sessptr != NULL)
-		xfree(sessptr);
+	free(sessptr);
 	return (-2);
 }
 
@@ -1080,7 +1111,7 @@ cmd_find_pane(struct cmd_ctx *ctx,
 
 	/* Lookup as pane id. */
 	if ((*wpp = cmd_lookup_paneid(arg)) != NULL) {
-		s = cmd_pane_session(ctx, *wpp, &wl);
+		s = cmd_window_session(ctx, (*wpp)->window, &wl);
 		if (sp != NULL)
 			*sp = s;
 		return (wl);
@@ -1113,7 +1144,7 @@ cmd_find_pane(struct cmd_ctx *ctx,
 			goto lookup_string;
 	}
 
-	xfree(winptr);
+	free(winptr);
 	return (wl);
 
 lookup_string:
@@ -1123,7 +1154,7 @@ lookup_string:
 		goto error;
 	}
 
-	xfree(winptr);
+	free(winptr);
 	return (wl);
 
 no_period:
@@ -1149,7 +1180,7 @@ lookup_window:
 	return (wl);
 
 error:
-	xfree(winptr);
+	free(winptr);
 	return (NULL);
 }
 
@@ -1213,34 +1244,85 @@ cmd_template_replace(char *template, const char *s, int idx)
 	return (buf);
 }
 
-/* Return the default path for a new pane. */
+/*
+ * Return the default path for a new pane, using the given path or the
+ * default-path option if it is NULL. Several special values are accepted: the
+ * empty string or relative path for the current pane's working directory, ~
+ * for the user's home, - for the session working directory, . for the tmux
+ * server's working directory. The default on failure is the session's working
+ * directory.
+ */
 const char *
-cmd_get_default_path(struct cmd_ctx *ctx)
+cmd_get_default_path(struct cmd_ctx *ctx, const char *cwd)
 {
-	const char		*cwd;
 	struct session		*s;
-	struct window_pane	*wp;
 	struct environ_entry	*envent;
+	const char		*root;
+	char			 tmp[MAXPATHLEN];
+	struct passwd		*pw;
+	int			 n;
+	size_t			 skip;
+	static char		 path[MAXPATHLEN];
 
 	if ((s = cmd_current_session(ctx, 0)) == NULL)
 		return (NULL);
 
-	cwd = options_get_string(&s->options, "default-path");
-	if ((cwd[0] == '~' && cwd[1] == '\0') || !strcmp(cwd, "$HOME")) {
-		envent = environ_find(&global_environ, "HOME");
-		if (envent != NULL && *envent->value != '\0')
-			return envent->value;
-		cwd = "";
-	}
-	if (*cwd == '\0') {
-		if (ctx->cmdclient != NULL && ctx->cmdclient->cwd != NULL)
-			return (ctx->cmdclient->cwd);
-		if (ctx->curclient != NULL) {
-			wp = s->curw->window->active;
-			if ((cwd = osdep_get_cwd(wp->pid)) != NULL)
-				return (cwd);
+	if (cwd == NULL)
+		cwd = options_get_string(&s->options, "default-path");
+
+	skip = 1;
+	if (strcmp(cwd, "$HOME") == 0 || strncmp(cwd, "$HOME/", 6) == 0) {
+		/* User's home directory - $HOME. */
+		skip = 5;
+		goto find_home;
+	} else if (cwd[0] == '~' && (cwd[1] == '\0' || cwd[1] == '/')) {
+		/* User's home directory - ~. */
+		goto find_home;
+	} else if (cwd[0] == '-' && (cwd[1] == '\0' || cwd[1] == '/')) {
+		/* Session working directory. */
+		root = s->cwd;
+		goto complete_path;
+	} else if (cwd[0] == '.' && (cwd[1] == '\0' || cwd[1] == '/')) {
+		/* Server working directory. */
+		if (getcwd(tmp, sizeof tmp) != NULL) {
+			root = tmp;
+			goto complete_path;
 		}
 		return (s->cwd);
+	} else if (*cwd == '/') {
+		/* Absolute path. */
+		return (cwd);
+	} else {
+		/* Empty or relative path. */
+		if (ctx->cmdclient != NULL && ctx->cmdclient->cwd != NULL)
+			root = ctx->cmdclient->cwd;
+		else if (ctx->curclient != NULL && s->curw != NULL)
+			root = osdep_get_cwd(s->curw->window->active->fd);
+		else
+			return (s->cwd);
+		skip = 0;
+		if (root != NULL)
+			goto complete_path;
 	}
-	return (cwd);
+
+	return (s->cwd);
+
+find_home:
+	envent = environ_find(&global_environ, "HOME");
+	if (envent != NULL && *envent->value != '\0')
+		root = envent->value;
+	else if ((pw = getpwuid(getuid())) != NULL)
+		root = pw->pw_dir;
+	else
+		return (s->cwd);
+
+complete_path:
+	if (root[skip] == '\0') {
+		strlcpy(path, root, sizeof path);
+		return (path);
+	}
+	n = snprintf(path, sizeof path, "%s/%s", root, cwd + skip);
+	if (n > 0 && (size_t)n < sizeof path)
+		return (path);
+	return (s->cwd);
 }
