@@ -1,4 +1,4 @@
-/* $Id: job.c 2553 2011-07-09 09:42:33Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -20,6 +20,7 @@
 #include <sys/socket.h>
 
 #include <fcntl.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
@@ -116,19 +117,19 @@ job_free(struct job *job)
 	log_debug("free job %p: %s", job, job->cmd);
 
 	LIST_REMOVE(job, lentry);
-	xfree(job->cmd);
+	free(job->cmd);
 
 	if (job->freefn != NULL && job->data != NULL)
 		job->freefn(job->data);
 
 	if (job->pid != -1)
 		kill(job->pid, SIGTERM);
-	if (job->fd != -1)
-		close(job->fd);
 	if (job->event != NULL)
 		bufferevent_free(job->event);
+	if (job->fd != -1)
+		close(job->fd);
 
-	xfree(job);
+	free(job);
 }
 
 /* Job buffer error callback. */
