@@ -1,4 +1,4 @@
-/* $Id: names.c 2553 2011-07-09 09:42:33Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -20,6 +20,7 @@
 
 #include <ctype.h>
 #include <libgen.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
@@ -36,7 +37,8 @@ queue_window_name(struct window *w)
 	tv.tv_sec = 0;
 	tv.tv_usec = NAME_INTERVAL * 1000L;
 
-	evtimer_del(&w->name_timer);
+	if (event_initialized(&w->name_timer))
+		evtimer_del(&w->name_timer);
 	evtimer_set(&w->name_timer, window_name_callback, w);
 	evtimer_add(&w->name_timer, &tv);
 }
@@ -48,10 +50,16 @@ window_name_callback(unused int fd, unused short events, void *data)
 	struct window	*w = data;
 	char		*name, *wname;
 
-	queue_window_name(w);	/* XXX even if the option is off? */
-	if (!options_get_number(&w->options, "automatic-rename"))
+	if (w->active == NULL)
 		return;
 
+	if (!options_get_number(&w->options, "automatic-rename")) {
+		if (event_initialized(&w->name_timer))
+			event_del(&w->name_timer);
+		return;
+	}
+	queue_window_name(w);
+
 	if (w->active->screen != &w->active->base)
 		name = NULL;
 	else
@@ -69,22 +77,20 @@ window_name_callback(unused int fd, unused short events, void *data)
 			wname = parse_window_name(name + 1);
 		else
 				wname = parse_window_name(name);
-		xfree(name);
+		free(name);
 	}
 
 	if (w->active->fd == -1) {
 		xasprintf(&name, "%s[dead]", wname);
-		xfree(wname);
+		free(wname);
 		wname = name;
 	}
 
-	if (strcmp(wname, w->name) == 0)
-		xfree(wname);
-	else {
-		xfree(w->name);
-		w->name = wname;
+	if (strcmp(wname, w->name)) {
+		window_set_name(w, wname);
 		server_status_window(w);
 	}
+	free(wname);
 }
 
 char *
@@ -120,6 +126,6 @@ parse_window_name(const char *in)
 	if (*name == '/')
 		name = basename(name);
 	name = xstrdup(name);
-	xfree(copy);
+	free(copy);
 	return (name);
 }
