@@ -1,4 +1,4 @@
-/* $Id: options-table.c 2669 2012-01-21 19:36:40Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2011 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -47,6 +47,9 @@ const char *options_table_status_keys_list[] = {
 const char *options_table_status_justify_list[] = {
 	"left", "centre", "right", NULL
 };
+const char *options_table_status_position_list[] = {
+	"top", "bottom", NULL
+};
 const char *options_table_bell_action_list[] = {
 	"none", "any", "current", NULL
 };
@@ -270,6 +273,11 @@ const struct options_table_entry session_options_table[] = {
 	  .default_num = KEYC_NONE,
 	},
 
+	{ .name = "renumber-windows",
+	  .type = OPTIONS_TABLE_FLAG,
+	  .default_num = 0
+	},
+
 	{ .name = "repeat-time",
 	  .type = OPTIONS_TABLE_NUMBER,
 	  .minimum = 0,
@@ -358,6 +366,12 @@ const struct options_table_entry session_options_table[] = {
 	  .default_num = 10
 	},
 
+	{ .name = "status-position",
+	  .type = OPTIONS_TABLE_CHOICE,
+	  .choices = options_table_status_position_list,
+	  .default_num = 1
+	},
+
 	{ .name = "status-right",
 	  .type = OPTIONS_TABLE_STRING,
 	  .default_str = "\"#22T\" %H:%M %d-%b-%y"
@@ -455,6 +469,21 @@ const struct options_table_entry window_options_table[] = {
 	  .default_num = 1
 	},
 
+
+	{ .name = "c0-change-trigger",
+	  .type = OPTIONS_TABLE_NUMBER,
+	  .default_num = 250,
+	  .minimum = 0,
+	  .maximum = USHRT_MAX
+	},
+
+	{ .name = "c0-change-interval",
+	  .type = OPTIONS_TABLE_NUMBER,
+	  .default_num = 100,
+	  .minimum = 1,
+	  .maximum = USHRT_MAX
+	},
+
 	{ .name = "clock-mode-colour",
 	  .type = OPTIONS_TABLE_COLOUR,
 	  .default_num = 4
@@ -480,6 +509,13 @@ const struct options_table_entry window_options_table[] = {
 	  .default_num = 0
 	},
 
+	{ .name = "layout-history-limit",
+	  .type = OPTIONS_TABLE_NUMBER,
+	  .minimum = 1,
+	  .maximum = USHRT_MAX,
+	  .default_num = 20
+	},
+
 	{ .name = "main-pane-height",
 	  .type = OPTIONS_TABLE_NUMBER,
 	  .minimum = 1,
@@ -574,6 +610,21 @@ const struct options_table_entry window_options_table[] = {
 	  .default_num = 0 /* overridden in main() */
 	},
 
+	{ .name = "window-status-activity-attr",
+	  .type = OPTIONS_TABLE_ATTRIBUTES,
+	  .default_num = GRID_ATTR_REVERSE
+	},
+
+	{ .name = "window-status-activity-bg",
+	  .type = OPTIONS_TABLE_COLOUR,
+	  .default_num = 8
+	},
+
+	{ .name = "window-status-activity-fg",
+	  .type = OPTIONS_TABLE_COLOUR,
+	  .default_num = 8
+	},
+
 	{ .name = "window-status-bell-attr",
 	  .type = OPTIONS_TABLE_ATTRIBUTES,
 	  .default_num = GRID_ATTR_REVERSE
@@ -604,21 +655,6 @@ const struct options_table_entry window_options_table[] = {
 	  .default_num = 8
 	},
 
-	{ .name = "window-status-activity-attr",
-	  .type = OPTIONS_TABLE_ATTRIBUTES,
-	  .default_num = GRID_ATTR_REVERSE
-	},
-
-	{ .name = "window-status-activity-bg",
-	  .type = OPTIONS_TABLE_COLOUR,
-	  .default_num = 8
-	},
-
-	{ .name = "window-status-activity-fg",
-	  .type = OPTIONS_TABLE_COLOUR,
-	  .default_num = 8
-	},
-
 	{ .name = "window-status-attr",
 	  .type = OPTIONS_TABLE_ATTRIBUTES,
 	  .default_num = 0
@@ -659,6 +695,16 @@ const struct options_table_entry window_options_table[] = {
 	  .default_str = "#I:#W#F"
 	},
 
+	{ .name = "window-status-separator",
+	  .type = OPTIONS_TABLE_STRING,
+	  .default_str = " "
+	},
+
+	{ .name = "wrap-search",
+	  .type = OPTIONS_TABLE_FLAG,
+	  .default_num = 1
+	},
+
 	{ .name = "xterm-keys",
 	  .type = OPTIONS_TABLE_FLAG,
 	  .default_num = 0
@@ -722,3 +768,36 @@ options_table_print_entry(
 	}
 	return (out);
 }
+
+/* Find an option. */
+int
+options_table_find(
+    const char *optstr, const struct options_table_entry **table,
+    const struct options_table_entry **oe)
+{
+	static const struct options_table_entry	*tables[] = {
+		server_options_table,
+		window_options_table,
+		session_options_table
+	};
+	const struct options_table_entry	*oe_loop;
+	u_int					 i;
+
+	for (i = 0; i < nitems(tables); i++) {
+		for (oe_loop = tables[i]; oe_loop->name != NULL; oe_loop++) {
+			if (strncmp(oe_loop->name, optstr, strlen(optstr)) != 0)
+				continue;
+
+			/* If already found, ambiguous. */
+			if (*oe != NULL)
+				return (-1);
+			*oe = oe_loop;
+			*table = tables[i];
+
+			/* Bail now if an exact match. */
+			if (strcmp((*oe)->name, optstr) == 0)
+				break;
+		}
+	}
+	return (0);
+}
