@@ -1,4 +1,4 @@
-/* $Id: osdep-darwin.c 2647 2011-12-09 16:37:29Z nicm $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2009 Joshua Elsasser <josh@elsasser.org>
@@ -20,12 +20,13 @@
 #include <sys/sysctl.h>
 
 #include <event.h>
+#include <libproc.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
 char			*osdep_get_name(int, char *);
-char			*osdep_get_cwd(pid_t);
+char			*osdep_get_cwd(int);
 struct event_base	*osdep_event_init(void);
 
 #define unused __attribute__ ((unused))
@@ -50,8 +51,22 @@ osdep_get_name(int fd, unused char *tty)
 }
 
 char *
-osdep_get_cwd(pid_t pid)
+osdep_get_cwd(int fd)
 {
+	static char 			wd[PATH_MAX];
+	struct proc_vnodepathinfo	pathinfo;
+	pid_t				pgrp;
+	int				ret;
+
+	if ((pgrp = tcgetpgrp(fd)) == -1)
+		return (NULL);
+
+	ret = proc_pidinfo(
+	    pgrp, PROC_PIDVNODEPATHINFO, 0, &pathinfo, sizeof pathinfo);
+	if (ret == sizeof pathinfo) {
+		strlcpy(wd, pathinfo.pvi_cdir.vip_path, sizeof wd);
+		return (wd);
+	}
 	return (NULL);
 }
 
