@@ -1,4 +1,4 @@
-/* $Id: screen-write.c 2666 2012-01-21 19:31:59Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,6 +18,7 @@
 
 #include <sys/types.h>
 
+#include <stdlib.h>
 #include <string.h>
 
 #include "tmux.h"
@@ -102,8 +103,8 @@ screen_write_cstrlen(int utf8flag, const char *fmt, ...)
 
 	size = screen_write_strlen(utf8flag, "%s", msg2);
 
-	xfree(msg);
-	xfree(msg2);
+	free(msg);
+	free(msg2);
 
 	return (size);
 }
@@ -141,7 +142,7 @@ screen_write_strlen(int utf8flag, const char *fmt, ...)
 		}
 	}
 
-	xfree(msg);
+	free(msg);
 	return (size);
 }
 
@@ -209,13 +210,17 @@ screen_write_vnputs(struct screen_write_ctx *ctx, ssize_t maxlen,
 			if (maxlen > 0 && size + 1 > (size_t) maxlen)
 				break;
 
-			size++;
-			screen_write_putc(ctx, gc, *ptr);
+			if (*ptr == '\001')
+				gc->attr ^= GRID_ATTR_CHARSET;
+			else {
+				size++;
+				screen_write_putc(ctx, gc, *ptr);
+			}
 			ptr++;
 		}
 	}
 
-	xfree(msg);
+	free(msg);
 }
 
 /* Write string, similar to nputs, but with embedded formatting (#[]). */
@@ -285,7 +290,7 @@ screen_write_cnputs(struct screen_write_ctx *ctx,
 		}
 	}
 
-	xfree(msg);
+	free(msg);
 }
 
 /* Parse an embedded style of the form "fg=colour,bg=colour,bright,...". */
@@ -878,6 +883,18 @@ screen_write_mousemode_on(struct screen_write_ctx *ctx, int mode)
 	s->mode |= mode;
 }
 
+/* Set bracketed paste mode. */
+void
+screen_write_bracketpaste(struct screen_write_ctx *ctx, int state)
+{
+	struct screen	*s = ctx->s;
+
+	if (state)
+		s->mode |= MODE_BRACKETPASTE;
+	else
+		s->mode &= ~MODE_BRACKETPASTE;
+}
+
 /* Line feed. */
 void
 screen_write_linefeed(struct screen_write_ctx *ctx, int wrapped)
@@ -1063,7 +1080,7 @@ screen_write_cell(struct screen_write_ctx *ctx,
 	screen_write_initctx(ctx, &ttyctx, 1);
 
 	/* If in insert mode, make space for the cells. */
-	if (s->mode & MODE_INSERT && s->cx <= screen_size_x(s) - width) {
+	if ((s->mode & MODE_INSERT) && s->cx <= screen_size_x(s) - width) {
 		xx = screen_size_x(s) - s->cx - width;
 		grid_move_cells(s->grid, s->cx + width, s->cx, s->cy, xx);
 		insert = 1;
