@@ -1,4 +1,4 @@
-/* $Id: server-window.c 2665 2012-01-21 19:30:07Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -19,6 +19,7 @@
 #include <sys/types.h>
 
 #include <event.h>
+#include <stdlib.h>
 #include <unistd.h>
 
 #include "tmux.h"
@@ -56,9 +57,6 @@ server_window_loop(void)
 				server_status_session(s);
 			TAILQ_FOREACH(wp, &w->panes, entry)
 				server_window_check_content(s, wl, wp);
-
-			if (!(s->flags & SESSION_UNATTACHED))
-				w->flags &= ~(WINDOW_BELL|WINDOW_ACTIVITY);
 		}
 	}
 }
@@ -76,46 +74,27 @@ server_window_check_bell(struct session *s, struct winlink *wl)
 		return (0);
 	if (s->curw != wl || s->flags & SESSION_UNATTACHED)
 		wl->flags |= WINLINK_BELL;
+	if (s->flags & SESSION_UNATTACHED)
+		return (1);
+	if (s->curw->window == wl->window)
+		w->flags &= ~WINDOW_BELL;
 
+	visual = options_get_number(&s->options, "visual-bell");
 	action = options_get_number(&s->options, "bell-action");
-	switch (action) {
-	case BELL_ANY:
-		if (s->flags & SESSION_UNATTACHED)
-			break;
-		visual = options_get_number(&s->options, "visual-bell");
-		for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
-			c = ARRAY_ITEM(&clients, i);
-			if (c == NULL || c->session != s)
-				continue;
-			if (!visual) {
-				tty_bell(&c->tty);
-				continue;
-			}
-			if (c->session->curw->window == w) {
-				status_message_set(c, "Bell in current window");
-				continue;
-			}
-			status_message_set(c, "Bell in window %u",
-			    winlink_find_by_window(&s->windows, w)->idx);
+	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+		c = ARRAY_ITEM(&clients, i);
+		if (c == NULL || c->session != s)
+			continue;
+		if (!visual) {
+			tty_bell(&c->tty);
+			continue;
 		}
-		break;
-	case BELL_CURRENT:
-		if (s->flags & SESSION_UNATTACHED)
-			break;
-		visual = options_get_number(&s->options, "visual-bell");
-		for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
-			c = ARRAY_ITEM(&clients, i);
-			if (c == NULL || c->session != s)
-				continue;
-			if (c->session->curw->window != w)
-				continue;
-			if (!visual) {
-				tty_bell(&c->tty);
-				continue;
-			}
+		if (c->session->curw->window == w)
 			status_message_set(c, "Bell in current window");
+		else if (action == BELL_ANY) {
+			status_message_set(c, "Bell in window %u",
+				winlink_find_by_window(&s->windows, w)->idx);
 		}
-		break;
 	}
 
 	return (1);
@@ -129,6 +108,9 @@ server_window_check_activity(struct session *s, struct winlink *wl)
 	struct window	*w = wl->window;
 	u_int		 i;
 
+	if (s->curw->window == wl->window)
+		w->flags &= ~WINDOW_ACTIVITY;
+
 	if (!(w->flags & WINDOW_ACTIVITY) || wl->flags & WINLINK_ACTIVITY)
 		return (0);
 	if (s->curw == wl && !(s->flags & SESSION_UNATTACHED))
@@ -217,6 +199,9 @@ server_window_check_content(
 	char		*found, *ptr;
 
 	/* Activity flag must be set for new content. */
+	if (s->curw->window == w)
+		w->flags &= ~WINDOW_ACTIVITY;
+
 	if (!(w->flags & WINDOW_ACTIVITY) || wl->flags & WINLINK_CONTENT)
 		return (0);
 	if (s->curw == wl && !(s->flags & SESSION_UNATTACHED))
@@ -227,7 +212,7 @@ server_window_check_content(
 		return (0);
 	if ((found = window_pane_search(wp, ptr, NULL)) == NULL)
 		return (0);
-	xfree(found);
+	free(found);
 
 	if (options_get_number(&s->options, "bell-on-alert"))
 		ring_bell(s);
