@@ -1,4 +1,4 @@
-/* $Id: tmux.c 2669 2012-01-21 19:36:40Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -73,11 +73,10 @@ logfile(const char *name)
 {
 	char	*path;
 
-	log_close();
 	if (debug_level > 0) {
 		xasprintf(&path, "tmux-%s-%ld.log", name, (long) getpid());
-		log_open_file(debug_level, path);
-		xfree(path);
+		log_open(debug_level, path);
+		free(path);
 	}
 }
 
@@ -245,7 +244,7 @@ main(int argc, char **argv)
 	quiet = flags = 0;
 	label = path = NULL;
 	login_shell = (**argv == '-');
-	while ((opt = getopt(argc, argv, "28c:df:lL:qS:uUvV")) != -1) {
+	while ((opt = getopt(argc, argv, "28c:Cdf:lL:qS:uUvV")) != -1) {
 		switch (opt) {
 		case '2':
 			flags |= IDENTIFY_256COLOURS;
@@ -256,32 +255,34 @@ main(int argc, char **argv)
 			flags &= ~IDENTIFY_256COLOURS;
 			break;
 		case 'c':
-			if (shell_cmd != NULL)
-				xfree(shell_cmd);
+			free(shell_cmd);
 			shell_cmd = xstrdup(optarg);
 			break;
+		case 'C':
+			if (flags & IDENTIFY_CONTROL)
+				flags |= IDENTIFY_TERMIOS;
+			else
+				flags |= IDENTIFY_CONTROL;
+			break;
 		case 'V':
 			printf("%s %s\n", __progname, VERSION);
 			exit(0);
 		case 'f':
-			if (cfg_file != NULL)
-				xfree(cfg_file);
+			free(cfg_file);
 			cfg_file = xstrdup(optarg);
 			break;
 		case 'l':
 			login_shell = 1;
 			break;
 		case 'L':
-			if (label != NULL)
-				xfree(label);
+			free(label);
 			label = xstrdup(optarg);
 			break;
 		case 'q':
 			quiet = 1;
 			break;
 		case 'S':
-			if (path != NULL)
-				xfree(path);
+			free(path);
 			path = xstrdup(optarg);
 			break;
 		case 'u':
@@ -300,8 +301,6 @@ main(int argc, char **argv)
 	if (shell_cmd != NULL && argc != 0)
 		usage();
 
-	log_open_tty(debug_level);
-
 	if (!(flags & IDENTIFY_UTF8)) {
 		/*
 		 * If the user has set whichever of LC_ALL, LC_CTYPE or LANG
@@ -363,7 +362,7 @@ main(int argc, char **argv)
 		}
 		xasprintf(&cfg_file, "%s/%s", home, DEFAULT_CFG);
 		if (access(cfg_file, R_OK) != 0 && errno == ENOENT) {
-			xfree(cfg_file);
+			free(cfg_file);
 			cfg_file = NULL;
 		}
 	}
@@ -385,16 +384,15 @@ main(int argc, char **argv)
 		/* -L or default set. */
 		if (label != NULL) {
 			if ((path = makesocketpath(label)) == NULL) {
-				log_warn("can't create socket");
+				fprintf(stderr, "can't create socket\n");
 				exit(1);
 			}
 		}
 	}
-	if (label != NULL)
-		xfree(label);
+	free(label);
 	if (realpath(path, socket_path) == NULL)
 		strlcpy(socket_path, path, sizeof socket_path);
-	xfree(path);
+	free(path);
 
 #ifdef HAVE_SETPROCTITLE
 	/* Set process title. */
