@@ -1,4 +1,4 @@
-/* $Id: window-copy.c 2646 2011-12-06 18:50:26Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -240,17 +240,16 @@ window_copy_free(struct window_pane *wp)
 	if (wp->fd != -1)
 		bufferevent_enable(wp->event, EV_READ|EV_WRITE);
 
-	if (data->searchstr != NULL)
-		xfree(data->searchstr);
-	xfree(data->inputstr);
+	free(data->searchstr);
+	free(data->inputstr);
 
 	if (data->backing != &wp->base) {
 		screen_free(data->backing);
-		xfree(data->backing);
+		free(data->backing);
 	}
 	screen_free(&data->screen);
 
-	xfree(data);
+	free(data);
 }
 
 void
@@ -805,14 +804,14 @@ window_copy_key_numeric_prefix(struct window_pane *wp, int key)
 
 	key &= KEYC_MASK_KEY;
 	if (key < '0' || key > '9')
-		return 1;
+		return (1);
 
 	if (data->numprefix >= 100) 	/* no more than three digits */
-		return 0;
+		return (0);
 	data->numprefix = data->numprefix * 10 + key - '0';
 
 	window_copy_redraw_lines(wp, screen_size_y(s) - 1, 1);
-	return 0;
+	return (0);
 }
 
 /* ARGSUSED */
@@ -822,7 +821,7 @@ window_copy_mouse(
 {
 	struct window_copy_mode_data	*data = wp->modedata;
 	struct screen			*s = &data->screen;
-	u_int				 i, old_cy;
+	u_int				 i;
 
 	if (m->x >= screen_size_x(s))
 		return;
@@ -835,10 +834,9 @@ window_copy_mouse(
 			for (i = 0; i < 5; i++)
 				window_copy_cursor_up(wp, 0);
 		} else if ((m->b & MOUSE_BUTTON) == MOUSE_2) {
-			old_cy = data->cy;
 			for (i = 0; i < 5; i++)
 				window_copy_cursor_down(wp, 0);
-			if (old_cy == data->cy)
+			if (data->oy == 0)
 				goto reset_mode;
 		}
 		return;
@@ -984,11 +982,12 @@ window_copy_search_up(struct window_pane *wp, const char *searchstr)
 	struct grid_cell	 	 gc;
 	size_t				 searchlen;
 	u_int				 i, last, fx, fy, px;
-	int				 utf8flag, n, wrapped;
+	int				 utf8flag, n, wrapped, wrapflag;
 
 	if (*searchstr == '\0')
 		return;
 	utf8flag = options_get_number(&wp->window->options, "utf8");
+	wrapflag = options_get_number(&wp->window->options, "wrap-search");
 	searchlen = screen_write_strlen(utf8flag, "%s", searchstr);
 
 	screen_init(&ss, searchlen, 1, 0);
@@ -1021,7 +1020,7 @@ retry:
 			break;
 		}
 	}
-	if (!n && !wrapped) {
+	if (wrapflag && !n && !wrapped) {
 		fx = gd->sx - 1;
 		fy = gd->hsize + gd->sy - 1;
 		wrapped = 1;
@@ -1041,11 +1040,12 @@ window_copy_search_down(struct window_pane *wp, const char *searchstr)
 	struct grid_cell	 	 gc;
 	size_t				 searchlen;
 	u_int				 i, first, fx, fy, px;
-	int				 utf8flag, n, wrapped;
+	int				 utf8flag, n, wrapped, wrapflag;
 
 	if (*searchstr == '\0')
 		return;
 	utf8flag = options_get_number(&wp->window->options, "utf8");
+	wrapflag = options_get_number(&wp->window->options, "wrap-search");
 	searchlen = screen_write_strlen(utf8flag, "%s", searchstr);
 
 	screen_init(&ss, searchlen, 1, 0);
@@ -1068,7 +1068,7 @@ window_copy_search_down(struct window_pane *wp, const char *searchstr)
 
 retry:
 	sgd = ss.grid;
-	for (i = fy + 1; i < gd->hsize + gd->sy; i++) {
+	for (i = fy + 1; i < gd->hsize + gd->sy + 1; i++) {
 		first = 0;
 		if (i == fy + 1)
 			first = fx;
@@ -1078,7 +1078,7 @@ retry:
 			break;
 		}
 	}
-	if (!n && !wrapped) {
+	if (wrapflag && !n && !wrapped) {
 		fx = 0;
 		fy = 0;
 		wrapped = 1;
@@ -1115,10 +1115,7 @@ window_copy_write_line(
 	char				 hdr[32];
 	size_t	 			 last, xoff = 0, size = 0;
 
-	memcpy(&gc, &grid_default_cell, sizeof gc);
-	colour_set_fg(&gc, options_get_number(oo, "mode-fg"));
-	colour_set_bg(&gc, options_get_number(oo, "mode-bg"));
-	gc.attr |= options_get_number(oo, "mode-attr");
+	window_mode_attrs(&gc, oo);
 
 	last = screen_size_y(s) - 1;
 	if (py == 0) {
@@ -1232,10 +1229,7 @@ window_copy_update_selection(struct window_pane *wp)
 		return (0);
 
 	/* Set colours. */
-	memcpy(&gc, &grid_default_cell, sizeof gc);
-	colour_set_fg(&gc, options_get_number(oo, "mode-fg"));
-	colour_set_bg(&gc, options_get_number(oo, "mode-bg"));
-	gc.attr |= options_get_number(oo, "mode-attr");
+	window_mode_attrs(&gc, oo);
 
 	/* Find top of screen. */
 	ty = screen_hsize(data->backing) - data->oy;
@@ -1378,7 +1372,7 @@ window_copy_copy_selection(struct window_pane *wp, int idx)
 
 	/* Don't bother if no data. */
 	if (off == 0) {
-		xfree(buf);
+		free(buf);
 		return;
 	}
 	off--;	/* remove final \n */
@@ -1629,7 +1623,7 @@ window_copy_cursor_up(struct window_pane *wp, int scroll_only)
 
 	oy = screen_hsize(data->backing) + data->cy - data->oy;
 	ox = window_copy_find_length(wp, oy);
-	if (ox != 0) {
+	if (data->cx != ox) {
 		data->lastcx = data->cx;
 		data->lastsx = ox;
 	}
@@ -1671,7 +1665,7 @@ window_copy_cursor_down(struct window_pane *wp, int scroll_only)
 
 	oy = screen_hsize(data->backing) + data->cy - data->oy;
 	ox = window_copy_find_length(wp, oy);
-	if (ox != 0) {
+	if (data->cx != ox) {
 		data->lastcx = data->cx;
 		data->lastsx = ox;
 	}
